clear();

z=300;
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);


s = poly(0,"s");
pa=0.0246580;
pb=7.9769475;
pc=280.08032;
G = syslin('c',[1],[pa*s*s+pb*s+pc]);
u = 300*ones(z,1);

y   = csim(u',t',G);
//plot(t,y(1,:),t,om);


disp("Mit zusätzlichem Integrator um Übertragungsfunktion für Winkel zu bekommen:");

GI = syslin('c',[1],[pa*s*s*s+pb*s*s+pc*s]);
yi = csim(u',t',GI);

//plot(t,yi(1,:),'re',t,phi,'gre',m1(:,1),-m1(:,2)+0.12,'blu');

rp=1200.0;
ri=16.0;

//R=rp+ri/s=(rp*s+ri)/s

R = syslin('c',[rp*s+ri],[s]);

H = R*GI;
Q = H/(1+H);

z=300;
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);
yq = csim(u',t',Q);

//plot(t,yq);

disp("Darstellung und Simulation von G im Zeitbereich");

Ag=[0,1;-pc/pa,-pb/pa];
Bg=[0;1/pa];

//Vergleichskontrolle der Eigenwerte:
zust=tf2ss(G);
Ag_test=zust(2);
disp(spec(Ag));
disp(spec(Ag_test));

disp("Simulationsmodell für G:");

function f = rechteSeiteG(t,yy)
    u_sim = 300.0;
    f = Ag*yy+Bg*u_sim; 
endfunction

y0 = [0,0]';
t0 = 0;
yg_sim  = ode(y0,t0,t,rechteSeiteG);

disp("O.K.: liegen ineinander");
//BEI BEDARF ANZEIGEN LASSEN:
//plot(t,yg_sim(1,:),'gre',t,y,'red--');

//------------------------------
//GI im Zeitbereich abbilden:

Agi=[   0, 1,    0;
        0, 0,    1;
        0,-pc/pa,-pb/pa];
Bgi=[0;0;1/pa];

disp("Simulationsmodell für GI:");

function f = rechteSeiteGI(t,yy)
    u_sim = 300.0;
    f = Agi*yy+Bgi*u_sim; 
endfunction

y0 = [0,0,0]';
t0 = 0;
ygi_sim  = ode(y0,t0,t,rechteSeiteGI);

disp("O.K.: liegen ineinander");
//BEI BEDARF ANZEIGEN LASSEN:
//plot(t,ygi_sim(1,:),'gre',t,yi,'red--');

//Auch hier die Eigenwerte vergleichen:
zusti=tf2ss(GI);
Agi_test=zusti(2);
disp(spec(Agi));
disp(spec(Agi_test));

//------------------------------
//GI mit PI-Regler im Zeitbereich abbilden:
//Jetzt Integral von phisoll - phi mitführen:
//(phisoll=0.2)
PHISOLL=0.2;
Agii=[   0,-1, 0,    0;
         0, 0, 1,    0;
         0, 0, 0,    1;
         0, 0,-pc/pa,-pb/pa];

Bgii=[1,0,0,0;
      0,0,0,0;
      0,0,0,0;
      0,0,0,1/pa];

rp=1200.0;
ri=16.0;

u_begrenzt_tot=0; //wird später berechnet, muß aber hier deklariert werden, da in rechteSeiteGIR gebraucht.

function f = rechteSeiteGIR(t,yy)
    w       = PHISOLL-yy(2);
    w_integ = yy(1); 

    u_begrenzt = rp*w + ri*w_integ;

    u_begrenzt = u_begrenzt_tot; //um Totzeit verzögertes Signal verwenden.


    if u_begrenzt>500 then
        u_begrenzt=500;
    end

    if u_begrenzt<-500 then
        u_begrenzt=-500;
    end

    u_sim   = [PHISOLL;0;0;u_begrenzt]; 
    f = Agii*yy+Bgii*u_sim; 
endfunction

y0 = [0,0,0,0]'; 
t0 = 0;
ygir_sim  = ode(y0,t0,t,rechteSeiteGIR);

//plot(t,ygir_sim(2,:),'gre',t,yq,'red--');


//-----------------------------------
//Definition Runge-Kutta-Integrator

modell=rechteSeiteGIR;
function yneu=ruku(y,t,dt)
       k1=modell(t,y);
       k2=modell(t+0.5*dt,y+0.5.*dt.*k1);
       k3=modell(t+0.5*dt,y+0.5.*dt.*k2);
       k4=modell(t+dt,y+dt.*k3);
       yneu=y+dt.*(k1+2.*k2+2.*k3+k4)./6;
endfunction

//-----------------------------------
//Optimierung:

talt=t;
ygir_sim_alt = ygir_sim;

z=900; //Erhöhung nötig, sonst RuKu zu ungenau!
u  = 0.2*ones(z,1);
t  = linspace(0,3,z);
dt = t(2)-t(1);

Tt = 0.0150984; //Identifizierte Totzeit in Sekunden.

//Anzahl der Simulationsschritte, bis die Totzeit vorbei ist:
anztt = round(Tt/dt);


u_merk = [0];

function err=berechneFehler(x)
    rp=x(1);
    ri=x(2);

    y0 = [0,0,0,0]'; 
    t0 = 0;

    //vorher:
    //ygir_sim  = ode(y0,t0,t,rechteSeiteGIR);

    //jetzt:
    //Simulation mit Runge-Kutta
    yalt = y0;
    ygir_sim = yalt;

    u_begrenzt_aktuell = 0;
    u_merk = [0];

    index_umerk = 1;

    for i=1:1:z-1
        takt = t(i);
        yneu=ruku(yalt,takt,dt);
        yalt=yneu;
        ygir_sim=[ygir_sim,yalt];

        //Stellsignal rekonstruieren:
        w       = PHISOLL-yalt(2);
        w_integ = yalt(1); 

        u_begrenzt_aktuell = rp*w + ri*w_integ;
        if u_begrenzt_aktuell>500 then
            u_begrenzt_aktuell=500;
        end

        if u_begrenzt_aktuell<-500 then
            u_begrenzt_aktuell=-500;
        end

        //In Array merken:
        u_merk = [u_merk,u_begrenzt_aktuell];

        //Anfangen aus den gemerkten Werten zu lesen, wenn die Totzeit vorbei ist:
        if i>=anztt
             u_begrenzt_tot = u_merk(index_umerk);
             index_umerk=index_umerk+1;
        end
    end

    //Achtung: phi ist der zweite Zustand (der erste ist das Integral von -phi).
    err = sum((ygir_sim(2,1:z)-u(1:z,1)').*(ygir_sim(2,1:z)-u(1:z,1)'));
    //err = u_merk; //Tests
endfunction

rp=1200.0;
ri=16.0;

x0=[rp;ri];

[fopt, xopt, gopt] = optim(list(NDcost,berechneFehler), "b", x0.*0.001, x0.*1000.0, x0);

rp=xopt(1);
ri=xopt(2);
y0 = [0,0,0,0]'; 
t0 = 0;



//ygir_simopt  = ode(y0,t0,t,rechteSeiteGIR);
//------- Kopie aus Fehlerfunktion ---------

//    rp=x(1);
//    ri=x(2);

    y0 = [0,0,0,0]'; 
    t0 = 0;

    //vorher:
    //ygir_sim  = ode(y0,t0,t,rechteSeiteGIR);

    //jetzt:
    //Simulation mit Runge-Kutta
    yalt = y0;
    ygir_sim = yalt;

    u_begrenzt_aktuell = 0;
    u_merk = [0];

    index_umerk = 1;

    for i=1:1:z-1
        takt = t(i);
        yneu=ruku(yalt,takt,dt);
        yalt=yneu;
        ygir_sim=[ygir_sim,yalt];

        //Stellsignal rekonstruieren:
        w       = PHISOLL-yalt(2);
        w_integ = yalt(1); 

        u_begrenzt_aktuell = rp*w + ri*w_integ;
        if u_begrenzt_aktuell>500 then
            u_begrenzt_aktuell=500;
        end

        if u_begrenzt_aktuell<-500 then
            u_begrenzt_aktuell=-500;
        end

        //In Array merken:
        u_merk = [u_merk,u_begrenzt_aktuell];

        //Anfangen aus den gemerkten Werten zu lesen, wenn die Totzeit vorbei ist:
        if i>=anztt
             u_begrenzt_tot = u_merk(index_umerk);
             index_umerk=index_umerk+1;
        end
    end


//--------Ende Kopie -----------------------


plot(t,ygir_sim(2,:),'blu');

disp(rp);
disp(ri);



